@@ -11,13 +11,13 @@
 # April 2010: Functions for LU, QR, SVD, Schur and Cholesky decompositions were
 # moved to their own files.  Still in this file are functions for eigenstuff
 # and for the Hessenberg form.
- 
+
 __all__ = ['eig','eigh','eig_banded','eigvals','eigvalsh', 'eigvals_banded',
            'hessenberg']
 
 import numpy
 from numpy import array, asarray_chkfinite, asarray, diag, zeros, ones, \
-        isfinite, inexact, nonzero, iscomplexobj, cast
+        isfinite, inexact, nonzero, iscomplexobj, cast, flatnonzero, conj
 
 # Local imports
 from scipy.linalg import calc_lwork
@@ -28,20 +28,17 @@ from blas import get_blas_funcs
 
 _I = cast['F'](1j)
 
-def _make_complex_eigvecs(w, vin, cmplx_tcode):
-    v = numpy.array(vin, dtype=cmplx_tcode)
-    #ind = numpy.flatnonzero(numpy.not_equal(w.imag,0.0))
-    ind = numpy.flatnonzero(numpy.logical_and(numpy.not_equal(w.imag, 0.0),
-                            numpy.isfinite(w)))
-    vnew = numpy.zeros((v.shape[0], len(ind)>>1), cmplx_tcode)
-    vnew.real = numpy.take(vin, ind[::2],1)
-    vnew.imag = numpy.take(vin, ind[1::2],1)
-    count = 0
-    conj = numpy.conjugate
-    for i in range(len(ind)/2):
-        v[:, ind[2*i]] = vnew[:, count]
-        v[:, ind[2*i+1]] = conj(vnew[:, count])
-        count += 1
+def _make_complex_eigvecs(w, vin, dtype):
+    """
+    Produce complex-valued eigenvectors from LAPACK DGGEV real-valued output
+    """
+    # - see LAPACK man page DGGEV at ALPHAI
+    v = numpy.array(vin, dtype=dtype)
+    m = (w.imag > 0)
+    m[:-1] |= (w.imag[1:] < 0) # workaround for LAPACK bug, cf. ticket #709
+    for i in flatnonzero(m):
+        v.imag[:,i] = vin[:,i+1]
+        conj(v[:,i], v[:,i+1])
     return v
 
 def _geneig(a1, b, left, right, overwrite_a, overwrite_b):
@@ -492,7 +489,7 @@ def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
             select = 2
             vl, vu, il, iu = 0.0, 0.0, min(select_range), max(select_range)
             if min(il, iu) < 0 or max(il, iu) >= a1.shape[1]:
-                raise ValueError, 'select_range out of bounds'
+                raise ValueError('select_range out of bounds')
             max_ev = iu - il + 1
         else:  # 1, 'v', 'value'
             select = 1
